//****************************************************************************** //
// $Id: $
// 
// Original Author: <name> // Creation Date: <date>
// $Author: $
// $Date: $
// $Revision: $
//
//*****************************************************************************
//
// Description: Example code showing how to boot, configure the 10MHz reference clock and close down the RFSG and RFSA VST components.
//
// Copyright (c) 2005-20015, Aeroflex International Ltd.
//
//
//*****************************************************************************


#include "stdafx.h"
#include "niRFSG.h"
#include "niRFSA.h"
#include <stdio.h>
#include <iostream>

//****************************************************************************** 
// Function          : ErrorCheckRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSA VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSA(ViSession Vsa, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}	

	delete errMsg;
}

//****************************************************************************** 
// Function          : ErrorCheckRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSG VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSG(ViSession Vsg, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}		

	delete errMsg;
}

//****************************************************************************** 
// Function          : BootRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSA within the VST and returns a reference to
// the RFSA.
//******************************************************************************
unsigned long BootRFSA()
{
	ViStatus status;
	ViSession vsa = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSA session		
	status = niRFSA_init("RIO0", VI_TRUE, VI_TRUE, &vsa);					
	ErrorCheckRFSA(vsa,status);

	// Configure 10MHz reference
	status = niRFSA_ConfigureRefClock(vsa, NIRFSA_VAL_ONBOARD_CLOCK_STR, referenceClockRate);
	ErrorCheckRFSA(vsa,status);

	// Commit the settings to make clock settings active
	status = niRFSA_Commit(vsa);
	ErrorCheckRFSA(vsa,status);

	std::cout << "VST - RFSA Booted and Configured for on-board 10 MHz reference clock. : " << std::endl;			

	return vsa;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
unsigned long BootRFSG()
{
	ViStatus status;
	ViSession vsg = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSG session
	status = niRFSG_init("RIO0", VI_TRUE, VI_TRUE, &vsg);
	ErrorCheckRFSG(vsg,status);

	// Configure 10MHz reference
	status = niRFSG_ConfigureRefClock(vsg, NIRFSG_VAL_ONBOARD_CLOCK_STR, referenceClockRate); // onboard reference
	ErrorCheckRFSG(vsg,status);

	// Export reference clock to REF out connector (if required)
	status = niRFSG_ExportSignal(vsg, NIRFSG_VAL_REF_CLOCK, "", NIRFSG_VAL_REF_OUT_STR);
	ErrorCheckRFSG(vsg,status);

	// Commit the settings to make clock settings active
	status = niRFSG_Commit(vsg);
	ErrorCheckRFSG(vsg,status);

	std::cout << "VST - RFSG Booted and Configured for on-board 10 MHz reference clock. : " << std::endl;			

	return vsg;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSA(ViSession vsa)
{
	ViStatus status;

	// Close session
	status = niRFSA_close(vsa);
	ErrorCheckRFSA(vsa,status);
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSG(ViSession vsg)
{
	ViStatus status;

	// Close session
	status = niRFSG_close(vsg);
	ErrorCheckRFSG(vsg,status);
}

//****************************************************************************** 
// Function          : _tmain
// Globals Used      : None
// Globals Modified  : None
// Description       : Console App entry point.
//******************************************************************************
int _tmain(int argc, _TCHAR* argv[])
{
	ViSession vsg = VI_NULL;
	ViSession vsa = VI_NULL;

	std::cout << "VST Example 1 - Booting, Configuring and Closing" << std::endl;
	std::cout << "------------------------------------------------" << std::endl;
	std::cout << std::endl;
	std::cout << "Booting and configuring RFSA." << std::endl;
	vsa = BootRFSA();
	std::cout << "Booting and configuring RFSG." << std::endl;
	vsg = BootRFSG();
	std::cout << "Closing RFSA." << std::endl;
	CloseRFSA(vsa);
	std::cout << "Closing RFSG." << std::endl;
	CloseRFSG(vsg);
	std::cout << std::endl;
	std::cout << std::endl;
	std::cout << "Press Any Key To Exit!" << std::endl;
	std::getchar();
	return 0;
}

