//****************************************************************************** //
// $Id: $
// 
// Original Author: <name> // Creation Date: <date>
// $Author: $
// $Date: $
// $Revision: $
//
//*****************************************************************************
//
// Description: Example code showing how to configure and then perform a simple single frequency, input level capture.
//
// Copyright (c) 2005-20015, Aeroflex International Ltd.
//
//
//*****************************************************************************


#include "stdafx.h"
#include "niRFSG.h"
#include "niRFSA.h"
#include <math.h> /* log10 */
#include <stdio.h>
#include <iostream>

//****************************************************************************** 
// Function          : ErrorCheckRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSA VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSA(ViSession Vsa, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}		

	delete errMsg;
}

//****************************************************************************** 
// Function          : ErrorCheckRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSG VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSG(ViSession Vsg, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}		

	delete errMsg;
}

//****************************************************************************** 
// Function          : BootRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSA within the VST and returns a reference to
// the RFSA.
//******************************************************************************
unsigned long BootRFSA()
{
	ViStatus status;
	ViSession vsa = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSA session		
	status = niRFSA_init("RIO0", VI_TRUE, VI_TRUE, &vsa);					
	ErrorCheckRFSA(vsa,status);

	// Configure 10MHz reference
	status = niRFSA_ConfigureRefClock(vsa, NIRFSA_VAL_ONBOARD_CLOCK_STR, referenceClockRate);
	ErrorCheckRFSA(vsa,status);

	// Commit the settings to make clock settings active
	status = niRFSA_Commit(vsa);
	ErrorCheckRFSA(vsa,status);

	std::cout << "VST - RFSA Booted and Configured for on-board 10 MHz reference clock. : " << std::endl;			

	return vsa;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
unsigned long BootRFSG()
{
	ViStatus status;
	ViSession vsg = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSG session
	status = niRFSG_init("RIO0", VI_TRUE, VI_TRUE, &vsg);
	ErrorCheckRFSG(vsg,status);

	// Configure 10MHz reference
	status = niRFSG_ConfigureRefClock(vsg, NIRFSG_VAL_ONBOARD_CLOCK_STR, referenceClockRate); // onboard reference
	ErrorCheckRFSG(vsg,status);

	// Export reference clock to REF out connector (if required)
	status = niRFSG_ExportSignal(vsg, NIRFSG_VAL_REF_CLOCK, "", NIRFSG_VAL_REF_OUT_STR);
	ErrorCheckRFSG(vsg,status);

	// Commit the settings to make clock settings active
	status = niRFSG_Commit(vsg);
	ErrorCheckRFSG(vsg,status);

	std::cout << "VST - RFSG Booted and Configured for on-board 10 MHz reference clock. : " << std::endl;			

	return vsg;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSA(ViSession vsa)
{
	ViStatus status;

	// Close session
	status = niRFSA_close(vsa);
	ErrorCheckRFSA(vsa,status);
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSG(ViSession vsg)
{
	ViStatus status;

	// Close session
	status = niRFSG_close(vsg);
	ErrorCheckRFSG(vsg,status);
}

//****************************************************************************** 
// Function          : Capture
// Globals Used      : None
// Globals Modified  : None
// Description       : Captures IQ for a specified dwell time at a specified frequency and input level at a specified sample rate.
//******************************************************************************
void Capture(ViSession vsa, ViReal64 CarrierFrequency, ViReal64 ReferenceLevel, int NumSamples, ViReal64 SampleRate, float* iBuffer, float* qBuffer )
{
	ViStatus status;

	//
	// Configure VST for capture
	//
	status = niRFSA_ConfigureAcquisitionType(vsa, NIRFSA_VAL_IQ); // IQ capture mode
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureIQCarrierFrequency(vsa, VI_NULL, CarrierFrequency); // Frequency
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureReferenceLevel(vsa, VI_NULL, ReferenceLevel); // Reference level
	ErrorCheckRFSA(vsa,status);

	// Set sample rate to that required by the analysis library for the selected measurements
	status = niRFSA_ConfigureIQRate(vsa, VI_NULL, SampleRate);
	ErrorCheckRFSA(vsa,status);
	
	status = niRFSA_ConfigureNumberOfSamples(vsa, VI_NULL, VI_TRUE, NumSamples); // Number of samples
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureNumberOfRecords(vsa, VI_NULL, VI_TRUE, 1); // Number of records
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_DisableStartTrigger(vsa); // immediate trigger
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_Initiate(vsa); // start capture
	ErrorCheckRFSA(vsa,status);

	if(status == 0)
	{
		// Fetch IQ data
		NIComplexNumberF32 *capturedIQ = new NIComplexNumberF32[NumSamples];

		status = niRFSA_SetAttributeViInt32(vsa, VI_NULL, NIRFSA_ATTR_FETCH_RELATIVE_TO, NIRFSA_VAL_FIRST_SAMPLE);
		ErrorCheckRFSA(vsa,status);

		niRFSA_wfmInfo captureInfo;
		ViReal64 timeout = 1.0;
		
		status = niRFSA_FetchIQSingleRecordComplexF32(vsa, VI_NULL, 0, -1, timeout, capturedIQ, &captureInfo); // -1 = fetch all samples
		ErrorCheckRFSA(vsa,status);

		if(status == 0 && captureInfo.actualSamples == NumSamples)
		{
			// Fetch of IQ samples was successful			
			for(int i = 0; i < NumSamples; i++)
			{
				iBuffer[i] = capturedIQ[i].real;
				qBuffer[i] = capturedIQ[i].imaginary;
			}
		}
		else
		{
			std::cout << "An error occured during capture ( the number of samples returned does not match what was asked for )!" << std::endl;
		}

		// Free the allocated memory.
		delete capturedIQ;
	}
	else
	{
		std::cout << "An error occured during capture!" << std::endl;
	}
}

//****************************************************************************** 
// Function          : CalculateAveragePower
// Globals Used      : None
// Globals Modified  : None
// Description       : Calculates the average power over a specified subset of
// the I an Q samples passed to the method.
//******************************************************************************
double CalculateAveragePower(float *iData, float *qData, int startIndex, int finishIndex, double levelCorrection)
{
	int totalSamples;            
	double powerInWatts = 0.0;
	double AveragePowerResult = -100;    
	double power = 0.0;

	for (int index = startIndex; index < finishIndex; index++)
	{
		// As the IQ sometime contains a value for I and Q that is zero and computing I2 + Q2 of 0 causes some errors in the Pvt trace,
		// I cycle through the other adjacent I Q samples looking for a non-zero sample.
		if (iData[index] == 0)
		{
			if (index == 0)
			{
				iData[index] = 0.000000001f;
			}
			else
			{
				iData[index] = iData[index - 1];
			}
		}

		if (qData[index] == 0)
		{
			if (index == 0)
			{
				qData[index] = 0.000000001f;
			}
			else
			{
				qData[index] = qData[index - 1];
			}
		}                       

		power = iData[index] * iData[index] + qData[index] * qData[index]; // Power in watts.               

		powerInWatts += power;

	}

	totalSamples = finishIndex - startIndex; // Calculate the number of samples the average will be made over.
	powerInWatts = powerInWatts / totalSamples;  // take the average.            
	AveragePowerResult = 10 * log10(powerInWatts) +  levelCorrection; // Convert to dBm

	return AveragePowerResult;
}

//****************************************************************************** 
// Function          : _tmain
// Globals Used      : None
// Globals Modified  : None
// Description       : Console App entry point.
//******************************************************************************
int _tmain(int argc, _TCHAR* argv[])
{
	ViSession vsg = VI_NULL;
	ViSession vsa = VI_NULL;
	ViReal64 frequency = 2.e9; // 2 GHz.
	ViReal64 referenceLevel = 20; // 20 dBm
	ViReal64 dwellTime = 1e-3; // 1 ms.
	ViReal64 sampleRate = 40e6; // 40 MHz
	double averagePowerdBm;
	int numberOfSamples = (int)(dwellTime * sampleRate);
	float *iData = new float[numberOfSamples];
	float *qData = new float[numberOfSamples];

	std::cout << "VST Example 2 - Configure and perform basic capture using RFSA" << std::endl;
	std::cout << "--------------------------------------------------------------" << std::endl;
	std::cout << std::endl;
	std::cout << "Booting and configuring RFSA." << std::endl;
	vsa = BootRFSA();

	// Capture some IQ.
	Capture(vsa, frequency, referenceLevel, numberOfSamples, sampleRate, iData, qData);

	// Now Calculate the Average Power Over the Capture
	averagePowerdBm = CalculateAveragePower(iData,qData,0,numberOfSamples-1,0);

	std::cout << std::endl;
	std::cout << std::endl;
	std::cout << "Average Power of Capture : " << averagePowerdBm << " dBm" << std::endl;
	std::cout << std::endl;
	std::cout << std::endl;

	// Now free up the memory allocated to store I and Q.
	delete[] iData;
	delete[] qData;

	std::cout << "Closing RFSA." << std::endl;
	CloseRFSA(vsa);

	std::cout << std::endl;
	std::cout << std::endl;
	std::cout << "Press Any Key To Exit!" << std::endl;
	std::getchar();
	return 0;
}

