//****************************************************************************** //
// $Id: $
// 
// Original Author: <name> // Creation Date: <date>
// $Author: $
// $Date: $
// $Revision: $
//
//*****************************************************************************
//
// Description: Example code showing how to generate a CW tone or playback a waveform from the VST using RFSG.
//
// Copyright (c) 2005-20015, Aeroflex International Ltd.
//
//
//*****************************************************************************

#include "stdafx.h"
#include "niRFSG.h"
#include "niRFSA.h"
#include <stdio.h>
#include <vector>
#include <fstream>
#include <string>
#include <iostream>
#include <direct.h>

//****************************************************************************** 
// Function          : ErrorCheckRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSA VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSA(ViSession Vsa, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}		

	delete errMsg;
}

//****************************************************************************** 
// Function          : ErrorCheckRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSG VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSG(ViSession Vsg, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}	

	delete errMsg;
}

//****************************************************************************** 
// Function          : BootRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSA within the VST and returns a reference to
// the RFSA.
//******************************************************************************
unsigned long BootRFSA()
{
	ViStatus status;
	ViSession vsa = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSA session		
	status = niRFSA_init("RIO0", VI_TRUE, VI_TRUE, &vsa);					
	ErrorCheckRFSA(vsa,status);

	// Configure 10MHz reference
	status = niRFSA_ConfigureRefClock(vsa, NIRFSA_VAL_ONBOARD_CLOCK_STR, referenceClockRate);
	ErrorCheckRFSA(vsa,status);

	// Commit the settings to make clock settings active
	status = niRFSA_Commit(vsa);
	ErrorCheckRFSA(vsa,status);

	std::cout << "VST - RFSA Booted and Configured for 10 MHz reference clock. : " << std::endl;			

	return vsa;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
unsigned long BootRFSG()
{
	ViStatus status;
	ViSession vsg = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSG session
	status = niRFSG_init("RIO0", VI_TRUE, VI_TRUE, &vsg);
	ErrorCheckRFSG(vsg,status);

	// Configure 10MHz reference
	status = niRFSG_ConfigureRefClock(vsg, NIRFSG_VAL_ONBOARD_CLOCK_STR, referenceClockRate); // onboard reference
	ErrorCheckRFSG(vsg,status);

	// Export reference clock to REF out connector (if required)
	status = niRFSG_ExportSignal(vsg, NIRFSG_VAL_REF_CLOCK, "", NIRFSG_VAL_REF_OUT_STR);
	ErrorCheckRFSG(vsg,status);

	// Commit the settings to make clock settings active
	status = niRFSG_Commit(vsg);
	ErrorCheckRFSG(vsg,status);

	return vsg;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSA(ViSession vsa)
{
	ViStatus status;

	// Close session
	status = niRFSA_close(vsa);
	ErrorCheckRFSA(vsa,status);
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSG(ViSession vsg)
{
	ViStatus status;

	// Close session
	status = niRFSG_close(vsg);
	ErrorCheckRFSG(vsg,status);
}

//****************************************************************************** 
// Function          : ReadCsvFile
// Globals Used      : None
// Globals Modified  : None
// Description       : Reads in an IQ waveform specified as a .csv file and returns to filled in I and Q vectors.
//******************************************************************************
void ReadCsvFile(std::string fileName, std::vector<ViReal64> &I, std::vector<ViReal64> &Q)
{
	
	std::string valueString;
	double value;
	std::string::size_type sz; // alias of size_t
	std::string Idata;
	std::string Qdata;

	std::ifstream file( fileName ); // Open File stream.

	while ( file.good() == true )
	{	
		getline( file, valueString, '\n' ); // Read IQ
		Idata = valueString.substr(0, valueString.find(","));
		if ( Idata != "")
		{			
			value = std::stod(Idata,&sz); // Convert the string to a double.
					
			I.push_back(value); // Add the I value to the I Vector.

			Qdata = valueString.substr(Idata.length() + 1, valueString.length());

			if ( Qdata != "")
			{
				value = std::stod(Qdata,&sz); // Convert the string to a double.

				// Add the Q value to the Q Vector.
				Q.push_back(value);
			}
		}
	}

}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Generates a CW tone at a given frequency and level.
//******************************************************************************
void GenerateCwTone(ViSession vsg, ViReal64 Frequency, ViReal64 Level)
{
	ViStatus status;
	ViBoolean done;

	// Aborts generation & put VSG in configuration mode
	status = niRFSG_Abort(vsg);
	ErrorCheckRFSA(vsg,status);

	//
	// Configure all settings that require the VSG to be in configuration mode
	//

	// Set generation mode to CW
	status = niRFSG_ConfigureGenerationMode(vsg, NIRFSG_VAL_CW);
	ErrorCheckRFSA(vsg,status);

	// Disable start trigger, so generation will commence immediately after _Initiate() called
	status = niRFSG_DisableStartTrigger(vsg);
	ErrorCheckRFSA(vsg,status);

	// Configure frequency & level using higher level function
	status = niRFSG_ConfigureRF(vsg, Frequency, Level);
	ErrorCheckRFSA(vsg,status);

	// Note level can also be configured using low level attributes.
	status = niRFSG_SetAttributeViReal64(vsg,VI_NULL, NIRFSG_ATTR_POWER_LEVEL, Level);
	ErrorCheckRFSA(vsg,status);

	// Note frequency can also be configured using low level attributes.
	status = niRFSG_SetAttributeViReal64(vsg,VI_NULL, NIRFSG_ATTR_FREQUENCY, Frequency);
	ErrorCheckRFSA(vsg,status);

	// Enable output
	status = niRFSG_ConfigureOutputEnabled(vsg, VI_TRUE);
	ErrorCheckRFSA(vsg,status);

	// Note output state can also be configured using low level attributes.
	status = niRFSG_SetAttributeViBoolean(vsg,VI_NULL, NIRFSG_ATTR_OUTPUT_ENABLED, VI_TRUE);
	ErrorCheckRFSA(vsg,status);

	// Initiate generation (VSG now in generation mode)
	status = niRFSG_Initiate(vsg);
	ErrorCheckRFSA(vsg,status);

	// Check Generation Status. ( returns 0 (generation active) )
	status = niRFSG_CheckGenerationStatus(vsg, &done); 
	ErrorCheckRFSA(vsg,status);
}

void PlayIQWaveform(ViSession vsg, std::string fileName, ViReal64 frequency, ViReal64 level, ViReal64 sampleRate)
{
	ViStatus status;
	ViBoolean done;
	std::vector<ViReal64> I;
	std::vector<ViReal64> Q;

	// Load in am IQ waveform from file that is specified with a IQ pair per line deliminated with a ','
	ReadCsvFile(fileName, I,Q);
	ViInt32 numSamples = I.size();

	// Aborts generation & put VSG in configuration mode
	status = niRFSG_Abort(vsg);
	ErrorCheckRFSA(vsg,status);

	//
	// Configure all settings that require the VSG to be in configuration mode
	//

	// RMS mode will automatically rescale ARB files to maximise DAC dynamic range.
	// PEAK mode will treat the RF level as being the peak value, so the RMS value will usually be lower.  ARB files are not re-scaled.
	status = niRFSG_ConfigurePowerLevelType(vsg, NIRFSG_VAL_AVERAGE_POWER);
	ErrorCheckRFSA(vsg,status);

	// Clear All wavefroms so we don't get an error if the waveform is already loaded.
	status = niRFSG_ClearAllArbWaveforms (vsg);
	ErrorCheckRFSA(vsg,status);
	
	// Write Arb file to RFSG.
	status = niRFSG_WriteArbWaveform(vsg, "gsmtsc0", numSamples, &I[0], &Q[0], VI_FALSE); 
	ErrorCheckRFSA(vsg,status);	
	// Set ARB playback sample rate
	status = niRFSG_SetAttributeViReal64(vsg, VI_NULL, NIRFSG_ATTR_IQ_RATE, sampleRate);
	ErrorCheckRFSA(vsg,status);

	// Set generation mode to ARB Waveform.
	status = niRFSG_ConfigureGenerationMode(vsg, NIRFSG_VAL_ARB_WAVEFORM);
	ErrorCheckRFSA(vsg,status);

	// Configure the ARB to playback continuously or a finite numberof times.
	status = niRFSG_SetAttributeViBoolean(vsg, VI_NULL, NIRFSG_ATTR_ARB_WAVEFORM_REPEAT_COUNT_IS_FINITE, VI_FALSE); // VI_FALSE for continuous playback VI_TRUE to play back n-times.
	ErrorCheckRFSA(vsg,status);

	// Configure number of times ARB file is played
	status = niRFSG_SetAttributeViInt32(vsg, VI_NULL, NIRFSG_ATTR_ARB_WAVEFORM_REPEAT_COUNT, 1000); 
	ErrorCheckRFSA(vsg,status);

	// Disable start trigger, so generation will commence immediately after _Initiate() called
	status = niRFSG_DisableStartTrigger(vsg);
	ErrorCheckRFSA(vsg,status);

	// Configure frequency & level using higher level function
	status = niRFSG_ConfigureRF(vsg, frequency, level);
	ErrorCheckRFSA(vsg,status);

	// Enable output
	status = niRFSG_ConfigureOutputEnabled(vsg, VI_TRUE);
	ErrorCheckRFSA(vsg,status);

	// Enable software start trigger
	status = niRFSG_ConfigureSoftwareStartTrigger(vsg);
	ErrorCheckRFSA(vsg,status);

	// Initiate generation (VSG now in generation mode)
	status = niRFSG_Initiate(vsg);
	ErrorCheckRFSA(vsg,status);

	// Check Generation Status. ( returns 0 (generation active) )
	status = niRFSG_CheckGenerationStatus(vsg, &done); 
	ErrorCheckRFSA(vsg,status);

	// VSG now in generation mode but awaiting a software trigger to commence ARB playback
	status = niRFSG_SendSoftwareEdgeTrigger(vsg, NIRFSG_VAL_START_TRIGGER, ""); // send software trigger
	ErrorCheckRFSA(vsg,status);
}

//****************************************************************************** 
// Function          : _tmain
// Globals Used      : None
// Globals Modified  : None
// Description       : Console App entry point.
//******************************************************************************
int _tmain(int argc, _TCHAR* argv[])
{
	ViSession vsg = VI_NULL;
	ViSession vsa = VI_NULL;
	std::vector<ViReal64> I;
	std::vector<ViReal64> Q;
	ViReal64 sampleRate = 20e6; // 20 MHz.
	ViReal64 frequency	= 1e9;	// 1 GHz
	ViReal64 level		= -20;	// -20 dBm
	int myChoice = 0;

	

	std::cout << "VST Example 3 - Generate CW Tone or Playback IQ Waveform" << std::endl;
	std::cout << "--------------------------------------------------------" << std::endl;
	std::cout << std::endl;
	// Boot and configure the RFSG.
	std::cout << "Booting and configuring RFSG." << std::endl;
	vsg = BootRFSG();
	std::cout << std::endl;
	std::cout << "********** Main Menu **********" << std::endl;
	std::cout << "(1): Generate CW Tone" << std::endl;
	std::cout << "(2): Playback IQ Waveform" << std::endl;
	std::cout << "(3): Exit" << std::endl;
	
	
	do
	{
		std::cin >> myChoice;

		switch(myChoice)
		{
		case 1:
			GenerateCwTone(vsg, frequency,level);
			break;
		case 2:
			PlayIQWaveform(vsg,"gsmtsc0.csv",frequency,level,sampleRate);
			break;	
		case 3:
			break;
		default:
			std::cout << "ERROR! You have selected an invalid choice." << std::endl;
			break;
		}	
	}while(myChoice !=3);
	
	// Cleanup

	I.clear();
	Q.clear();
	
	return 0;
}

